#!/usr/bin/perl
use IO::Handle;
use em8300;


sub write_register {
  my $addr = shift;
  my $data = shift;
  
  if($debugio) {
    printf "Write 0x%04x to register 0x%x ",$data,$addr;
  }
  
  em8300_write($addr,$data);
  
  if($debugio) {
    printf ("I2C %x", (read_register(0x1f4d) & 0x1800) >> 8);
    print "\n";
  }
}

sub read_register {
  my $addr = shift;
  my $data;
  return em8300_read($addr,$data);
}


sub I2C_data {
  my $val = shift;
  write_register(0x1f4d,0x800 | ($val ? 8 : 0));
}

sub I2C_read_data {
  I2C_drivedata(0);
  my $val = (read_register(0x1f4d) & 0x800) ? 1 : 0;
  I2C_drivedata(1);
  return $val;
}

sub I2C_drivedata {
  my $val = shift;
  write_register(0x1f4e,0x800 | ($val ? 8 : 0));
}

sub I2C_clk {
  my $val = shift;
  if($useclk==2) {
    write_register(0x1f4d,0x2000 | ($val ? 0x20 : 0));
  } else {
    write_register(0x1f4d,0x1000 | ($val ? 0x10 : 0));
  }
}

sub I2C_out {
  my $data = shift;
  my $bits = shift;
  
  for(my $i=$bits-1; $i >= 0; $i--) {
    I2C_data($data & (1 << $i));
    I2C_clk(1);
    I2C_clk(0);

  }
}

sub I2C_in {
  my $bits = shift;
  my $data;
  
  for(my $i=$bits-1; $i >= 0; $i--) {
    $data |= I2C_read_data() << $i;
    I2C_clk(1);
    I2C_clk(0);
  }
  return $data;
}

sub I2C_start {
  I2C_data(1);
  I2C_clk(1);
  I2C_data(0);
  I2C_clk(0);
}

sub I2C_stop {
  I2C_clk(0);
  I2C_data(0);
  I2C_clk(1);
  I2C_data(1);
}


#
# I2c_write(slave_address, subaddress, data1, data2 ...)
#
sub I2C_write {
  $slaveaddr = shift;
  $subaddr = shift;
  
  &I2C_start;
  
  I2C_out($slaveaddr,7);	# slave address
  I2C_out(0,1);			# R/W
  &I2C_ack or return -1;	# Ack
  
  I2C_out($subaddr,8);		# slave address
  &I2C_ack or return -1;	# Ack
  
  while($#_ >= 0) {
    I2C_out(shift,8);		# data
    &I2C_ack or return -1;	# Ack
  }
  
  # Stopbit
  &I2C_stop;
  
  return 0;
}

sub I2C_ack {
    I2C_drivedata(0);
  for(my $count=0; $count < 256; $count++) {
    if(!I2C_read_data()) {
      I2C_drivedata(1);
      I2C_clk(1);
      I2C_clk(0);
      return 1;
    }
  }
  I2C_drivedata(1);
  return 0;
}

#
# I2c_read(slave_address, subaddress, # of bytes to read)
#
sub I2C_read {
  my $slaveaddr = shift;
  my $subaddr = shift;
  my $n = shift;
  my @data;
  
  &I2C_start;
  
  I2C_out($slaveaddr,7);	# slave address
  I2C_out(0,1);			# R/W
  &I2C_ack or return 0;	        # Ack

  
  I2C_out($subaddr,8);		# sub address
  &I2C_ack or return 0;	        # Ack
  
  # Startbit
  I2C_start;
  
  I2C_out($slaveaddr,7);	# slave address
  I2C_out(1,1);			# R/W
  &I2C_ack or return 0;	        # Ack
  
  for(my $i=0; $i < $n; $i++) {
    push(@data,I2C_in(8));	# data
    I2C_out(1,1);
  }
  
  &I2C_stop;
  
  return @data;
}

sub write_myst {
  my $data = shift;
  
  I2C_data(1);
  for(my $i=0; $i <8 ; $i++) {
    write_register(0x1f4d, 0x2000);
    I2C_data($data & 1);
    write_register(0x1f4d, 0x2020);
    $data >>= 1;
  }
  write_register(0x1f4d, 0x0200);
  write_register(0x1f4d, 0x0202);
}

sub reset {
  write_register(0x1f50,0x123);
  read_register(0x1c08);
  write_register(0x1f4d,0x3f3f);
  write_register(0x1f4e,0x3b3b);
  write_register(0x1f4d,0x0100);
  write_register(0x1f4d,0x0101);
  write_register(0x1f4d,0x0808);
}  

em8300_open;

STDOUT->autoflush(1);

print "EM8300>";
while(<>) {
  chomp;
  $_ = lc();
  if(/^e/) { last; };
  if(/^sw /) { 
    s/sw ([0-9a-f]+)/ write_myst(hex($1));/e;
  };
  if(/^w /) {
    s/w ([0-9a-f]+) ([0-9a-f]+)/ em8300_write(hex($1),hex($2));/e;
  } 
  if(/^r /) {
    s/r ([0-9a-f]+)/ printf ("0x%x\n",em8300_read(hex($1)));/e;
  } 

  if(/^status/) {
    $status = &em8300_getstatus;
    $tmp = $status;
    $tmp =~ s/Time elapsed: ([0-9]+) us/$time=$1/e;
    $tmp =~ s/Frames: ([0-9]+)/$frames=$1/e;
    $tmp =~ s/SCR: ([0-9]+)/$scr=$1/e;
    print $status;
    if($time) {
      print "FPS: ", $frames / ($time * 1e-6), "\n";
      print "SCRPS: ", $scr / ($time * 1e-6), "\n";
    }
    print "\n";
  } 

  if(/^ma /) { 
    printf "MA_Status  : %04x\n", em8300_read(0x111f);
    printf "MA_RdPtr   : %04x\n", em8300_read(0x1124) | (em8300_read(0x1125) << 16);
    printf "MA_WrPtr   : %04x\n", em8300_read(0x1127);
    printf "MA_PCIRdPtr: %04x\n", em8300_read(0x1130);
    printf "MA_PCIWrPtr: %04x\n", em8300_read(0x1131);
  };
  print "EM8300>"
}

em8300_close
